"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Task = void 0;
const tslib_1 = require("tslib");
const chalk_1 = tslib_1.__importDefault(require("chalk"));
class Task {
    //
    // Constructor.
    constructor() {
        //
        // Protected properties.
        this._configs = null;
        this._coreName = '';
        this._manager = null;
        this._overloads = 0;
        this.canOverload = false;
        this.interval = 5000;
        this.runAtStart = false;
        this.load();
    }
    //
    // Public methods.
    code() {
        return this._coreName;
    }
    coreRun() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            // istanbul ignore else
            if (yield this.coreShouldRun()) {
                this._overloads++;
                try {
                    yield this.run();
                }
                catch (err) {
                    console.error(chalk_1.default.yellow(`DRTools::Tasks: There was an error running task '${this.name()}'.`), err);
                }
                this._overloads--;
            }
        });
    }
    description() {
        return null;
    }
    getInterval() {
        return this.interval;
    }
    paused() {
        return this._manager !== null
            && !this._manager.stopped()
            && !this._manager.paused().includes(this.code());
    }
    running() {
        return this._overloads > 0;
    }
    setConfigs(configs) {
        this._configs = configs;
    }
    setCoreName(coreName) {
        this._coreName = coreName;
    }
    setManager(manager) {
        this._manager = manager;
    }
    shouldRunAtStart() {
        return this.runAtStart;
    }
    //
    // Protected methods.
    /* istanbul ignore next */
    coreShouldRun() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return (!this.running() || this.canOverload) && (yield this.shouldRun());
        });
    }
    /* istanbul ignore next */
    load() {
    }
    /* istanbul ignore next */
    shouldRun() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return true;
        });
    }
}
exports.Task = Task;
