"use strict";
/**
 * @file tenedor-child.ts
 * @author Alejandro D. Simi
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.TenedorChild = void 0;
const tslib_1 = require("tslib");
const exceptions_1 = require("./exceptions");
class TenedorChild {
    //
    // Construction.
    constructor(child) {
        //
        // Properties.
        this.childProcess = null;
        this.childProcess = child;
    }
    //
    // Public methods.
    ask(data) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.checkRunning();
            //
            // Creating listeners for the process. It assumes that the first message
            // sent by the child is the one and only that's going to be sent before
            // exiting.
            const childResponse = new Promise((resolve, reject) => {
                var _a;
                (_a = this.childProcess) === null || _a === void 0 ? void 0 : _a.once('message', (response) => response.error ? reject(response.error) : resolve(response.data));
            });
            const childExit = new Promise((resolve, reject) => {
                var _a;
                (_a = this.childProcess) === null || _a === void 0 ? void 0 : _a.once('exit', (code) => reject(`The process exited unexpectedly with code ${code}.`));
            });
            //
            // Sending a question to the child and listening for it's response.
            yield new Promise((resolve, reject) => {
                var _a;
                (_a = this.childProcess) === null || _a === void 0 ? void 0 : _a.send(data, (error) => error ? reject(error) : resolve());
            });
            return Promise.race([childResponse, childExit]);
        });
    }
    child() {
        return this.childProcess;
    }
    kill(signal) {
        //
        // Whether the child finished correctly or not, it has to be stopped.
        // istanbul ignore else
        if (this.childProcess && this.childProcess.exitCode === null) {
            this.childProcess.kill(signal);
            this.childProcess = null;
        }
    }
    tell(data) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.checkRunning();
            //
            // Sending a question to the child and listening for it's response.
            yield new Promise((resolve, reject) => {
                var _a;
                (_a = this.childProcess) === null || _a === void 0 ? void 0 : _a.send(data, (error) => error ? reject(error) : resolve());
            });
        });
    }
    //
    // Protected methods.
    checkRunning() {
        //
        // Does it have a valid child process?
        // istanbul ignore else
        if (!this.childProcess) {
            throw new exceptions_1.TenedorNoChildException(`Child process is not running.`);
        }
    }
}
exports.TenedorChild = TenedorChild;
