"use strict";
/**
 * @file tenedor.ts
 * @author Alejandro D. Simi
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.Tenedor = void 0;
const tslib_1 = require("tslib");
const child_process_1 = require("child_process");
const glob_1 = require("glob");
const tenedor_child_1 = require("./tenedor-child");
const exceptions_1 = require("./exceptions");
const fs_extra_1 = tslib_1.__importDefault(require("fs-extra"));
class Tenedor {
    //
    // Construction.
    // istanbul ignore next
    constructor() {
        throw new exceptions_1.TenedorException(`This is a factory-class, it's not possible to create instances of it.`);
    }
    //
    // Public methods.
    static Run(script, data) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let results = null;
            const tenedorChild = yield Tenedor.Spawn(script);
            //
            // Telling the child to start and listening for it's response.
            try {
                results = yield tenedorChild.ask(data);
            }
            finally {
                tenedorChild.kill();
            }
            return results;
        });
    }
    static Spawn(script) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            //
            // Does the script exists?
            let scriptPaths = [];
            if (fs_extra_1.default.existsSync(script)) {
                scriptPaths = [script];
            }
            else {
                scriptPaths = glob_1.glob.sync(`${script}.?s`);
            }
            // istanbul ignore else
            if (scriptPaths.length !== 1) {
                throw new exceptions_1.TenedorMissingScriptException(script, `'${script}' doesn't exist.`);
            }
            //
            // Spawning a new child.
            const child = child_process_1.fork(scriptPaths[0]);
            //
            // Checking that it actually launches.
            const childSpawn = new Promise((resolve, reject) => {
                child.once('spawn', () => resolve(true));
            });
            const childError = new Promise((resolve, reject) => {
                child.once('error', (err) => reject(err));
            });
            yield Promise.race([childSpawn, childError]);
            return new tenedor_child_1.TenedorChild(child);
        });
    }
}
exports.Tenedor = Tenedor;
